<?php

namespace Teknisa\DF\Controllers;

use Estoque\Entities\Lanctoestoq;
use Fiscal\Entities\Notafiscal;
use Fiscal\Entities\Itemproimpos;
use Teknisa\DF\Services\LancamentoNotafiscalService;
use Teknisa\DF\Services\ValidacoesEstoqueService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class LancamentoNotafiscalController extends \Teknisa\DF\Controllers\TeknisaCrud
{
    protected $lancamentoNotafiscalService;
    protected $validacoesEstoqueService;
    protected $nfEntradaService;

    public function __construct($crudService, $parameterBag, LancamentoNotafiscalService $lancamentoNotafiscalService, ValidacoesEstoqueService $validacoesEstoqueService, $environment,  $entityManager)
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->lancamentoNotafiscalService = $lancamentoNotafiscalService;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function montaCorpoNota (Request\Filter $request, Response $response) {
    	$requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];
        //$organizacao = $requestConditions[2]["value"];
        $entrada_saida = $requestConditions[2]["value"];
        $statusNota = $requestConditions[3]["value"];

        $arrayProdutos = $this->lancamentoNotafiscalService->buscaProdutos($filial, $lancamento);
        $arrayImpostosProdutos = $this->lancamentoNotafiscalService->buscaImpostosProdutos($filial, $lancamento, $entrada_saida, $statusNota);
        $arrayImpostos = $this->lancamentoNotafiscalService->buscaImpostos($filial, $lancamento, $entrada_saida, $statusNota);
        $arrayEncargos = $this->lancamentoNotafiscalService->buscaEncargos($filial, $lancamento);
        $arrayVencimentos = $this->lancamentoNotafiscalService->buscaVencimentos($filial, $lancamento, $entrada_saida);
        $arrayDadosAdicionais = $this->lancamentoNotafiscalService->buscaDadosAdicionais($filial, $lancamento);

        $arrayItensRateionf = $this->lancamentoNotafiscalService->buscaItensRateionf($filial, $lancamento);
        $arrayRateionf = $this->lancamentoNotafiscalService->buscaRateionf($filial, $lancamento, $arrayItensRateionf);
        
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens', $arrayProdutos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_imposto_item', $arrayImpostosProdutos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_imposto', $arrayImpostos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_encargo', $arrayEncargos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_vencimento', $arrayVencimentos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_dadosadic', $arrayDadosAdicionais));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens_rateionf', $arrayItensRateionf));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_rateionf', $arrayRateionf));
    }
    
    public function lancamento_nota_itens (Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];
        
        $arrayProdutos = $this->lancamentoNotafiscalService->buscaProdutos($filial, $lancamento);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens', $arrayProdutos));
    }

    public function getItensRateio(Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];
        
        $itensRateio = $this->lancamentoNotafiscalService->buscaItensRateionf($filial, $lancamento);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('itens_rateio', $itensRateio));
    }

    public function excluirNotaSaida(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            
            $filial = $conditions[0]['value'];
            $lancamento = $conditions[1]['value'];

            $filter = array(
                array(
                    "columnName" => "IDDOCREPLIC",
                    "operator"=> "=",
                    "value"=> $filial.$lancamento
                )  
            );
            
            $notasReplicadas = $this->crudService->getFilteredDataSet('notaReplicada_tipooperac', $filter);
            $notasReplicadas = $notasReplicadas->getRows();
            
            if($notasReplicadas) {
                foreach ($notasReplicadas as $notaReplic) {
                    $this->lancamentoNotafiscalService->procedureDeletaNf($notaReplic['CDFILIAL'], $notaReplic['NRLANCTONF']);
                }
            }
            
            $this->lancamentoNotafiscalService->deletaRelacNotaOrigem($filial, $lancamento);
            
            $this->lancamentoNotafiscalService->validaExclusaoNotaUtilizaAf($filial, $lancamento);

            $lanctoestoq = Lanctoestoq::findOneByNrlanctonfAndCdfilial($lancamento, $filial);
            
            $this->lancamentoNotafiscalService->verificaDevolucaoOl($filial, $lancamento);

            $this->lancamentoNotafiscalService->procedureDeletaNf($filial, $lancamento);

            if (is_object($lanctoestoq)) {
                $lancamentoEstoque = $lanctoestoq->getNrlancestq();
                $this->validacoesEstoqueService->setLanctoestoq($lanctoestoq);
                $this->validacoesEstoqueService->atualizaEstoque($filial, $lancamentoEstoque);
            }
            
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('excluir_nota_saida', array('success' => 'S')));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvarItem(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $item = $conditions[0]['value'];
            
            if(!isset($item['CDALMOXARIFE'])) $item['CDALMOXARIFE'] = ' ';
            else if($item['CDALMOXARIFE'] == null) $item['CDALMOXARIFE'] = ' ';
            
            if(!isset($item['CDLOCALESTOQ'])) $item['CDLOCALESTOQ'] = ' ';
            else if($item['CDLOCALESTOQ'] == null) $item['CDLOCALESTOQ'] = ' ';
            
            if(!isset($item['NRLOTEESTQ']))   $item['NRLOTEESTQ']   = ' ';
            else if($item['NRLOTEESTQ'] == null) $item['NRLOTEESTQ'] = ' ';
            
            if(!isset($item['NRSUBLOTE']))    $item['NRSUBLOTE']    = ' ';
            else if($item['NRSUBLOTE'] == null) $item['NRSUBLOTE'] = ' ';

            if(empty($item['CDSERVFATU'])) $item['CDSERVFATU'] =  null;
            if(empty($item['CDSERVFILI'])) $item['CDSERVFILI'] =  null;
            if(empty($item['DSINFADICPROD'])) $item['DSINFADICPROD'] = null;
            
            
            $this->lancamentoNotafiscalService->procedureGeraNotaFiscalProduto(
                $item['CDFILIAL'], $item['NRLANCTONF'], $item['CDPRODUTO'], $item['CDPRODEMIS'], $item['DSPRODEMIS'], $item['QTITEMPROD'], $item['VRUNITITEM'], $item['VRDESCITEM'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ'], 
                $item['NRLOTEESTQ'], $item['NRSUBLOTE'], $item['CDSERVFATU'], null, 'S', null, null, $item["CDSERVFILI"], $item['DSINFADICPROD']
            );
            
            $this->lancamentoNotafiscalService->salvaCFOPeCST($item['CDFILIAL'], $item['NRLANCTONF'], $item['NRSEQITPROD'], $item['CDFISOPEITEM'], $item['CDSITUTRIBPR']);

//            $this->alteraStatusNotafiscal($item['CDFILIAL'], $item['NRLANCTONF'], $item['NRORG'], 'D');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function alteraStatusNotafiscal($filial, $lancamento, $organizacao, $status){
        //@todo somente quando status nota !== 'D'
        $this->lancamentoNotafiscalService->updateStatusNotafiscal($filial, $lancamento, $organizacao, $status);
    }

    public function fecharNota(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();

            $cabecalho = $conditions[0]['value'];
            $vencimentos = $conditions[1]['value'];
            $forcaFechamento = isset($conditions[2])?   $conditions[2]['value'] : false;;
            
            $infoNotaFiscal = $this->lancamentoNotafiscalService->fechaNotaFiscal($cabecalho, $vencimentos, $forcaFechamento);

            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('fecharNota', $infoNotaFiscal));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error('Ocorreram erros ao realizar o fechamento da nota fiscal.', $e->getCode()));
            $erros = $this->lancamentoNotafiscalService->getErrosAvisos();
            if(empty($erros)) {
               $erros = array(array("CDERROAVISO" => "", "DSERROAVISO" => $e->getMessage(), "IDERROAVISO" => "B")); 
            }
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('errosFechamento', $erros));
        }
    }
    
    public function geraRateio(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $campos = array();
            foreach ($conditions as $campo){
                $campos[$campo['columnName']] = $campo['value'];
            }
            $infoRateio = $this->lancamentoNotafiscalService->geraRateio($campos['FILIAL'], $campos['LANCAMENTO'], $campos['IDENTRSAID'], $campos['RATEIONF'], $campos['GERARATAUT'], $campos['REALIZAMANUT']);
            //var_dump($infoRateio);die;
            //$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('geraRateio', $infoRateio));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        //$filial, $lancamento, $idEntrSaid,  $rateioNF, $autMan = 'N', $realizaManut = 'P'
        
    }

    public function calculaImpostoItemProduto(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $cabecalho = $rows[0];
            $item = $rows[1];
            
            $calculoImpostoProduto = $this->lancamentoNotafiscalService->calculaImpostoItemProduto($cabecalho, $item, false);

            $response->addDataset(new DataSet('calculoImpostoProduto', $calculoImpostoProduto));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvaItensAf(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $afSelecionadas = $rows;

            $arrayProdutos = $this->lancamentoNotafiscalService->salvaItensAf($afSelecionadas);

            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens', $arrayProdutos));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function replicacaoManual(Request\Dataset $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $notafiscal = $rows[0];
            $documentos = $rows[1];
            $itens      = $rows[2];
            $replicacaoManual = $this->lancamentoNotafiscalService->replicacaoManual($notafiscal, $documentos, $itens);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('replicacaoManual', $replicacaoManual));
        }
        catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function produtoLancamento(Request\Filter $request, Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $cdFilial = '';
        $cdSegMerc = '';
        
        foreach($conditions as &$condition) {
            if($condition["columnName"] == "CDFILIAL") {
                $cdFilial = $condition["value"];
            }
            if($condition["columnName"] == 'CDARVPROD|NMPRODUTO'){
                if($condition['operator'] == "MAPPED_LIKE_ALL"){
                    $condition['value'][0]['_ALL'] = str_replace('.', '', $condition['value'][0]['_ALL']);
                } else{
                    $condition['value'] = str_replace('.', '', $condition['value']);
                }
            }
        }
        
        $cdSegMerc = $this->lancamentoNotafiscalService->getSegMercFilial($cdFilial);
        if(isset($cdSegMerc[0])) {
            $cdSegMerc = $cdSegMerc[0]["CDSEGMER"];
        }
        
        if($cdSegMerc) {
            $filterSegmento = array("columnName" => "CDSEGMER", "operator" => "=", "value" => $cdSegMerc);
            array_push($conditions, $filterSegmento);
            $dataSourceName = 'prodfili_produto_segmercado';
        }
        else {
            $dataSourceName = 'prodfili_produto_semseg';
        }
        
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());

        $dataSet = new DataSet('produto_filial', $dataSet->getRows());
        $response->setDataSets(array($dataSet));
    }
    
    private function preparaFiltroCNPJ(&$conditions, $rowsName){
        foreach($conditions as $key => &$condition){
            if($condition['columnName'] == $rowsName){
                $caracteresEspeciais = array("-", ".", "/");
                if($condition['operator'] == "MAPPED_LIKE_ALL"){
                    $condition['value'][0]['_ALL'] = str_replace($caracteresEspeciais, '', $condition['value'][0]['_ALL']);
                } else{
                    $condition['value'] = str_replace($caracteresEspeciais, '', $condition['value']);
                }
            }
        }
        
        return $conditions;
    }
    
    public function getFornecedores(Request\Filter $request, Response $response) {
        $franqueadora = $this->lancamentoNotafiscalService->getFranqueadora();
        
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        
        $conditions = $this->preparaFiltroCNPJ($conditions, "NRINSJURFORN|NMRAZSOCFORN|NMFANTFORN");
        
        $filterCriteria = new FilterCriteria('', $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
        $request = new Request\Filter($filterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        
        
        if($franqueadora) {
            $this->dataSourceName = 'fornecedor_lancamento_notafiscal_filial';
            $request->getFilterCriteria()->addCondition("P_CDFILIAL", "=", $this->environment->getCdfilial());
            parent::find($request, $response);
        }
        else {
            $this->dataSourceName = 'fornecedor_lancamento_notafiscal';
            parent::find($request, $response);
        }
    }
    
    public function getClientes(Request\Filter $request, Response $response){
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        
        $conditions = $this->preparaFiltroCNPJ($conditions, 'NRINSJURCLIE|NMRAZSOCCLIE|NMFANTCLIE');
        $this->dataSourceName = 'cliente_lancamento_notafiscal';
        
        $filterCriteria = new FilterCriteria('cliente_lancamento_notafiscal', $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
        $request = new Request\Filter($filterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        
        parent::find($request, $response);
    }
    

    public function getCentcustRateio(Request\Filter $request, Response $response) {
        $this->dataSourceName = 'centcust_rateio';
    
        parent::find($request, $response);
        
        $rows = $response->getDataSets()[0]->getRows();
        
        if(count($rows) == 0) {
            
            $filterCriteria = $request->getFilterCriteria();
            $conditions = $filterCriteria->getConditions();
            
            foreach($conditions as &$condition) {
                if($condition["columnName"] == "CDFILIAL") {
                    $condition["value"] = "T";
                }
            }
            $filterCriteria = new FilterCriteria('centcust_rateio', $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
            $request = new Request\Filter($filterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::find($request, $response);
        }

    }
    

    public function getDefaultLancamentoNota(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $cdFilial = isset($row["CDFILIAL"]) ? $row["CDFILIAL"] : ''; 
        $idEntrSaidOp = isset($row["CDFILIAL"]) ? $row["CDFILIAL"] : 'E';
        
        $defaultFilter = $this->lancamentoNotafiscalService->getDefaultNovaNota($row["CDFILIAL"], $row["IDENTRSAIDOP"]);
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('defaultNovaNota', $defaultFilter));
    }
    
    public function copiarNF(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            if (isset($row["IDORIGDESTOP"])){
                $tipoDestino = $row["IDORIGDESTOP"];
            }else if(isset($row["IDORIGDESTNF"])) {
                 $tipoDestino = $row["IDORIGDESTNF"];
            }
            
            switch ($tipoDestino) {
            case "C":
                $cliforn = $row["CDCLIENTE"];
                break;
            case "F":
                 $cliforn = $row["CDFORNECED"];
                break;
            case "L":
                 $cliforn = $row["CDFILIRECE"];
                break;
            }
            
            $paramfilial = $this->getParamFilial($row["CDFILIAL"]);
            $atualizaestoque = $this->getAtualizaEstoque($paramfilial, $row["CDFILIAL"], $row["CDTIPOOPER"]);
            $row['DTENTRSAID'] = $row['DTLANCAMENNF'];

            if(isset($row["IDTPIJURTRAN"])){
                $proxLancamento = $this->lancamentoNotafiscalService->geraCorpoNota(
                    $row["CDFILIAL"],  $row["CDFILIEMIT"],$row["CDEMPRESA"] , $row["CDTIPOOPER"], $row["CDSERIE"], $cliforn, $row["IDSISTEMA"], $atualizaestoque, 
                    '', $row["IDTPIJURTRAN"], $row["NRINSJURTRAN"], $row["NRPLACAVEIC"], $row["SGUFVEIC"], $row["IDPGTOFRETE"], $row["DSQTDEVOLU"], $row["DSESPECVOLU"], $row["DSMARCAVOLU"], $row["DSNUMEVOLU"], '',
                    $row["DTENTRSAID"], $row["VRPBRUTOVOLU"], $row["VRPLIQUVOLU"], '', $row["DTLANCAMENNF"]);
            }else{
                $proxLancamento = $this->lancamentoNotafiscalService->geraCorpoNota(
                    $row["CDFILIAL"],  $row["CDFILIEMIT"],$row["CDEMPRESA"] , $row["CDTIPOOPER"], $row["CDSERIE"], $cliforn, $row["IDSISTEMA"], $atualizaestoque, 
                    '', '',  null ,null , null, null, null, null, null, $row["DSNUMEVOLU"], '',
                    $row["DTENTRSAID"], $row["VRPBRUTOVOLU"], $row["VRPLIQUVOLU"], '', $row["DTLANCAMENNF"]);
            } 
            $prxLanctoNF = $proxLancamento["P_PRXLANCTONF"];
            $prxLancEstq = $proxLancamento["P_PRXLANCESTQ"];
            
            $itens = $this->lancamentoNotafiscalService->getItensNFOrigem($row["CDFILIAL"],$row["NRLANCTONF"]);
            foreach($itens as $item){
                $lote = ''; 
                $subLote= '';
                
                if ($row["LOTEORIG"]  === 'S'){
                    $lote    = $item["NRLOTEESTQ"];
                    $subLote = $item["NRSUBLOTE"];
                }
                
                $this->lancamentoNotafiscalService->procedureGeraNotaFiscalProduto(
                    $row["CDFILIAL"], $prxLanctoNF, $item["CDPRODUTO"], $item["CDPRODEMIS"], $item["DSPRODEMIS"],  
                    $item["QTITEMPROD"], $item["VRUNITITEM"],  $item["VRDESCITEM"], '', '', $lote, $subLote, null,
                    $item["IDORIGEMPROD"], $item["IDATUESTPROD"],  0, $item["CDMARCA"], null);
                    
                    
                $idutildtproc = $this->getParamGeral()->getIdutildtproc();
                
                if($idutildtproc ==='S') {
                    $dtprocessamento = $paramfilial->getDtprocessa();
                    if(isset($dtprocessamento)  && $dtprocessamento != null){
                       $dtprocessamento =  $dtprocessamento->format('d/m/Y');
                    }else{
                         throw new \Exception("Data de processamento não encontrada.");
                    }
                }else {
                    $dtprocessamento = date('d/m/y');
                }
                
                $this->lancamentoNotafiscalService->procedureGeraFechamentoSaida( $row["CDFILIAL"], $prxLanctoNF,  $dtprocessamento, 
                                                                                    $prxLancEstq, '', 'S', 'N');
            }
            
            $impostObg =  $this->verifImpObrig( $row["CDFILIEMIT"], $row["CDFILIAL"], $prxLanctoNF);
            if(!empty($impostObg)){
              $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('errosCopiarNF', $this->erroImpoObg($impostObg)));
              throw new \Exception("Erro ao copiar Nota Fiscal.");
            }else{
                $connection->commit();
                $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('copiarNF', array('success' => 'Nota Fiscal copiada com sucesso')));  
            }

        }catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            $this->entityManager->rollback();
        }
        
        
    }
    
    public function getAtualizaEstoque($paramfilial, $cdfilial, $tipoOperacao){
        $atualizaestoque = 'S';

        if($paramfilial->getIdctrlestq() == 'N'){
            $atualizaestoque = 'N';
        } else {
            $operacao = $this->getTipooperac($tipoOperacao);  
            if($operacao->getIdatuesttpop() === 'N'){
                $atualizaestoque = 'N';
            }
        }
        
        return $atualizaestoque;
    }
    
    public function utilizaControleFiscal($paramfilial, $paramGeral) {
        if ($paramGeral->getIdutctrlfissaid() === 'S')
            return true;
        else if ($paramfilial->getIdutctrlfissaid() === 'S')
            return true;
        else 
            return false;
    }
    
    public function verifImpObrig( $cdfiliemit, $cdfilial, $prxLanctoNF) {
        $ProdutosSemImpObrig = array();
        if ($this->utilizaControleFiscal( $this->getParamFilial($cdfiliemit) , $this->getParamGeral())){
            $nf = $this->lancamentoNotafiscalService->getDadosNF($cdfilial, $prxLanctoNF);
            $impoObrig =  $this->lancamentoNotafiscalService->impostoObrig($nf[0]["IDENTRSAIDOP"], $nf[0]["CDTIPOOPER"]);
            foreach ($impoObrig as $imposto){
                if ($imposto["IDPRODNOTA"] = 'P'){
                    $impostoProduto = $this->lancamentoNotafiscalService->impostoProduto($cdfilial, $prxLanctoNF,$imposto["CDIMPOSTO"]);
                    if(isset($impostoProduto[0])){
                        $impostoProduto = $impostoProduto[0];
                        if(isset($ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]])){
                           $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["SGIMPOSTO"] .= ',' . $imposto["SGIMPOSTO"];
                        }else{
                            $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["ERROIMPOPROD"] = 'O produto não possui os impostos obrigatórios: '. $imposto["SGIMPOSTO"];
                            $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["NMPRODUTO"] = $impostoProduto["NMPRODUTO"];
                            $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["CDPRODUTO"] = $impostoProduto["CDPRODUTO"];
                            $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["CDARVPROD"] = $impostoProduto["CDARVPROD"];
                        }
                }
                }
            }
        }
        return $ProdutosSemImpObrig; 
    }
    
    public function getParamFilial($cdfilial){
       return  $this->entityManager->getRepository('Geral\Entities\Paramfilial')->findOneBy(array('nrorg' => $this->environment->getNrOrgTrab(), 'cdfilial' => $cdfilial));
    }
    
    public function getTipooperac($tipoOperacao){
        return $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Tipooperac')->findOneBy(
            array('identrsaidop' => 'S',
                  'cdtipooper' => $tipoOperacao));
    }
    
    public function getParamGeral(){
        return  $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Paramgeral')->findOneBy(array('nrorg' => $this->environment->getNrOrgTrab()));
    }
    
    public function erroImpoObg($impostObg){
            $produtosSemImposto = array();
            foreach($impostObg as $imposto){
               array_push($produtosSemImposto,$imposto);
            }
            return $produtosSemImposto;
    }
    
    public function produtoImportarLancamento(Request\Filter $request, Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $cdFilial = '';
        $cdSegMerc = '';
        
        foreach($conditions as &$condition) {
            if($condition["columnName"] == "CDFILIAL") {
                $cdFilial = $condition["value"];
            }
            if($condition["columnName"] == 'CDARVPROD|NMPRODUTO'){
                if($condition['operator'] == "MAPPED_LIKE_ALL"){
                    $condition['value'][0]['_ALL'] = str_replace('.', '', $condition['value'][0]['_ALL']);
                } else{
                    $condition['value'] = str_replace('.', '', $condition['value']);
                }
            }
        }
        
        $cdSegMerc = $this->lancamentoNotafiscalService->getSegMercFilial($cdFilial);
        
        if(isset($cdSegMerc[0])) {
            $cdSegMerc = $cdSegMerc[0]["CDSEGMER"];
        }
        
        if($cdSegMerc) {
            $dataSourceName = 'prodfili_produto_prodfornxml_produtoseg';
        }
        else {
            $dataSourceName = 'prodfili_produto_prodfornxml';
        }
        
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());

        $dataSet = new DataSet($dataSourceName, $dataSet->getRows());
        $response->setDataSets(array($dataSet));
    }
    
    
    

}