<?php

namespace Teknisa\DF\Modules\Saida\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;


class Grs15000ExportacaoNfFaturamentoSapFactory{
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    
    public function procedureExportacao($cdfilial, $nrlanctonf) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'EXPORTA_APURFATUR');
        $procedure->addParam(new Param('P_FILIAL',     Param::PARAM_INPUT, $cdfilial,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function updateNotaFiscal($cdfilial, $nrlanctonf) {
        $params = array(
            "P_CDFILIAL"   => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf
        );
        $sql = "UPDATE NOTAFISCAL
                    SET IDBLOQUEADO = 'S'
                WHERE CDFILIAL     = :P_CDFILIAL
                    AND NRLANCTONF = :P_NRLANCTONF";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateAviso($idaviso, $crcredito, $dsobservcred, $cdunicoaviso, $cdunico) {
        $params = array(
            "IDAVISO"      => $idaviso,
            "CRCREDITO"    => $crcredito,
            "DSOBSERVCRED" => $dsobservcred,
            "CDUNICOAVISO" => $cdunicoaviso,
            "P_CDUNICO"    => $cdunico
        );
        $sql = "UPDATE USR_SAP_APURFATUR
                SET IDAVISO      = :IDAVISO,
                   CRCREDITO     = :CRCREDITO,
                   DSOBSERVCRED  = :DSOBSERVCRED,
                   CDUNICOAVISO  = :CDUNICOAVISO
                WHERE CDUNICO    = :P_CDUNICO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function salvarNotaCredito($cdunicoaviso, $cdunico) {
        $params = array(
            "CDUNICOAVISO" => $cdunicoaviso,
            "P_CDUNICO"    => $cdunico
        );
        $sql = "UPDATE USR_SAP_APURFATUR
                SET CDUNICOAVISO = :CDUNICOAVISO
                WHERE CDUNICO    = :P_CDUNICO";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getFilterExportSap($cdfilial) {
        $sql = "SELECT CDFILIAL,
                  NMFILIAL,
                  F.CDEMPRESA,
                  E.NMEMPRESA
                FROM FILIAL F,
                  EMPRESA E
                WHERE CDFILIAL  = :P_CDFILIAL
                AND F.CDEMPRESA = E.CDEMPRESA";
                
        $params = array(
            "P_CDFILIAL" => $cdfilial
        );  
                
        $std = $this->connection->prepare($sql);    
        $std->execute($params);
        return $std->fetch();
    }
}
