<?php
namespace Teknisa\DF\Services\Emissao\Banco;

use Geral\Util\DateUtil;
use Geral\Entities\Banco;
use Geral\Util\NovoCodigo;
use Geral\Entities\Contcorr;
use Teknisa\DF\Services\Emissao\GeraEmissaoBoletoService;

/**
 * Class GeraEmissaoBoletoCaixaService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco
 */
class GeraEmissaoBoletoCaixaService extends GeraEmissaoBoletoService
{

    /**
     * Tamanho do operador da linha digitavel de nosso numero.
     */
    const TAMANHO_NOSSO_NUMERO = 17;
    const MODULO_DIGITO_VERIFICADOR_NOSSONUM = 11;
    const MODULO_DIGITO_VERIFICADOR_LINHADIGIT = 10;
    const FATOR_VENCIMENTO = '07/10/1997';

    private $codigoDeBarrasLinhaDigitavel;


    public function getNossoNumeroComDigito()
    {
        $codigoReduzido = $this->getCodigoReduzido();
        $cdcontador = self::PREFIXO_CDCONTADOR . $codigoReduzido;
        $nossoNumero = \Geral\Util\NovoCodigo::geraCodigo(
            $this->boleto->getNrorg(), $cdcontador, 1, self::TAMANHO_NOSSO_NUMERO
        );

        return $nossoNumero . $this->getDigitoVerificador($nossoNumero, self::SEQUENCIA_NOSSO_NUMERO);
    }


    /** Função que monta a Linha Digitável do boleto de acordo com o Layout do Banco Itaú.
     * @param $nossoNumero
     * @return string
     */

    public function getLinhaDigitavel($nossoNumero)
    {

        $grupo1 = substr($this->codigoDeBarrasLinhaDigitavel, 0, 4);
        $grupo1 .= substr($this->codigoDeBarrasLinhaDigitavel, 19, 5);
        $grupo1 .= $this->getVerificadorGruposLinhaDigitavel($grupo1);

        $grupo2 = substr($this->codigoDeBarrasLinhaDigitavel, 24, 10);
        $grupo2 .= $this->getVerificadorGruposLinhaDigitavel($grupo2);


        $grupo3 = substr($this->codigoDeBarrasLinhaDigitavel, 34, 10);
        $grupo3 .= $this->getVerificadorGruposLinhaDigitavel($grupo3);

        $grupo4 = $this->getDigitoCodigoBarras();

        $grupo5 = $this->getFatorVencimento();
        $grupo5 .= $this->getValorNominal();

        return self::concatLinhaDigitavel($grupo1, $grupo2, $grupo3, $grupo4, $grupo5);
    }

    public function getCodigoDeBarras($nossoNumero)
    {
        try {
            $codigoDeBarras = $this->getIdentificadorBanco();
            $codigoDeBarras .= self::MOEDA_REAL;
            //Esta string será substituída pelo código do dígito verificador
            $codigoDeBarras .= "_SUBSTITUICAO_DV_";
            $codigoDeBarras .= $this->getFatorVencimento();
            $codigoDeBarras .= $this->getValorNominal();
            $codigoDeBarras .= $this->getCdcedenteban();
            $codigoDeBarras .= $this->getDigitoVerificadorCedenteCaixa();
            $codigoDeBarras .= substr($nossoNumero, 2, 3);
            $codigoDeBarras .= substr($nossoNumero, 0, 1);
            $codigoDeBarras .= substr($nossoNumero, 5, 3);
            $codigoDeBarras .= substr($nossoNumero, 1, 1);
            $codigoDeBarras .= substr($nossoNumero, 8, 9);
            $codigoDeBarras .= $this->getDigitoCampoLivre(
                $this->getCdcedenteban().$this->getDigitoVerificadorCedenteCaixa().substr($nossoNumero, 2, 3).substr($nossoNumero, 0, 1).
                substr($nossoNumero, 5, 3).substr($nossoNumero, 1, 1).substr($nossoNumero, 8, 9)
            );

            $digito = $this->getDigitoVerificadorCaixa(str_replace("_SUBSTITUICAO_DV_", "", $codigoDeBarras));
            $codigoDeBarras = str_replace("_SUBSTITUICAO_DV_", $digito, $codigoDeBarras);

            $this->codigoDeBarrasLinhaDigitavel = $codigoDeBarras;

            return $codigoDeBarras;

        } catch (\Exception $e) {
            \Geral\Util\TratamentoDeMensagens::exibirMensagens($e);
        }
    }

    private function getValorNominal()
    {
        $valorBoleto = (string)$this->getValorBoleto();
        $valorBoleto = str_replace(',', '.', $valorBoleto);
        $valorBoleto = number_format($valorBoleto, 2, '.', '');
        $arrayExclusao = array('.', ',');
        $valorBoleto = str_replace($arrayExclusao, "", $valorBoleto);
        $valorBoleto = str_pad($valorBoleto, 10, "0", STR_PAD_LEFT);
        return $valorBoleto;
    }

    private function getCodigoCedente()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '0000';
        } else {
            $codigoCedente = substr($this->getCdcedenteban(), 0, 4);
        }

        return $codigoCedente;
    }

    private function getIdentificadorBanco()
    {
        $identificadorBanco = $this->getCdbanccnab();
        return $identificadorBanco;
    }

    private function getFatorVencimento()
    {
        $datavenc = DateUtil::truncateData($this->getDataVencimento());
        $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO));
        $fatorDeVencimento = DateUtil::getQtdeDiasEntre($dataFatorVencimento, $datavenc);
        $fatorDeVencimento = str_pad($fatorDeVencimento, 4, "0", STR_PAD_LEFT);
        return $fatorDeVencimento;
    }

    private function getVerificadorGruposLinhaDigitavel($linhaDigitavel)
    {
        $linhaDigitavel = strrev($linhaDigitavel);
        $peso = 2;
        $total = 0;
        for ($i = 0; $i <= strlen($linhaDigitavel); $i++) {
            $result = (int)substr($linhaDigitavel, $i, 1) * $peso;
            if ($result > 9) {
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1, 1);
            } else {
                $total += ((int)substr($linhaDigitavel, $i, 1) * $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2);

        }
        $digito = $total % 10;
        $soma = 0;
        if (strlen($digito) > 1) {
            for ($i = 0; $i < strlen($digito); $i++) {
                $soma += (int)substr($digito, $i, 1);
            }
        } else
            $soma = 10 - $digito;

        $soma = $soma > 9 ? 0 : $soma;

        return $soma;
    }

    public function getRelatorioBoleto($nrorg, $cdreducont)
    {

        /** @var $contcorr \Geral\Entities\Contcorr */
        $contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($nrorg, $cdreducont);
        $contcorr->getcdbanco();

        /** @var $dsarqbolcomp Banco */
        $dsarqbolcomp = \Geral\Entities\Banco::findOneByCdbanco($contcorr->getcdbanco());
        $string = $dsarqbolcomp->getDsarqbolcomp();
        $tam = strlen($string);
        $tam_b = strrpos($string, '\\');
        $ext = -($tam - strrpos($string, '.'));
        if ($tam_b === false) {
            return NULL;
        } else
            return ($string_p = substr($string, ++$tam_b, $ext));
    }

    private function concatLinhaDigitavel($grupo1, $grupo2, $grupo3, $grupo4, $grupo5)
    {

        $linhadigitavel = substr($grupo1, 0, 5) . '.' . substr($grupo1, 5) . ' ';

        $linhadigitavel .= substr($grupo2, 0, 5) . '.' . substr($grupo2, 5) . ' ';

        $linhadigitavel .= substr($grupo3, 0, 5) . '.' . substr($grupo3, 5) . ' ';

        $linhadigitavel .= $grupo4 . ' ';

        $linhadigitavel .= $grupo5;

        return $linhadigitavel;
    }

    protected function getDigitoCampoLivre($sequencia)
    {
        // Inverte o nosso numero.
        $sequencia = strrev($sequencia);
        $peso = 2;
        $acumulador = 0;

        // Este loop percorrera todos os caracteres da variavel $nossonumero
        for ($i = 0; $i < strlen($sequencia); $i++) {
            $acumulador += ((int)substr($sequencia, $i, 1) * $peso);
            $peso = $this->incrementaValorPeso($peso, 2, 9);
        }

        if ($acumulador < 11) {
            $digito = 11 - $acumulador;
        } else {
            $digito = $acumulador % self::MODULO_DIGITO_VERIFICADOR;
            $digito = 11 - $digito;
        }

        $digito = $digito > 9 ? 0 : $digito;

        return $digito;
    }

    public function chamaRelatorio()
    {
        // TODO: Implement chamaRelatorio() method.
    }
}
